/*
Quake PSP port.
Copyright (C) 2005 Peter Mackay

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <pspaudiolib.h>
#include <pspaudio.h>

extern "C"
{
#include "../quakedef.h"
}

static const size_t	channelCount	= 2;
static const size_t	frequency		= 44100;
static const size_t	bufferSize		= 16384;

struct Sample
{
	short left;
	short right;
};

static Sample	sampleBuffer[bufferSize];
static int		samplesWritten;

static void audioCallback(void* buffer, unsigned int samplesToWrite, void *userdata)
{
	Sample* const destination = static_cast<Sample*> (buffer);

	// Going to wrap past the end of the input buffer?
	const unsigned int samplesBeforeEndOfInput = bufferSize - samplesWritten;
	if (samplesToWrite > samplesBeforeEndOfInput)
	{
		// Yes, so copy the first chunk.
		memcpy(&destination[0], &sampleBuffer[samplesWritten], samplesBeforeEndOfInput * sizeof(Sample));

		// Copy the second chunk.
		const unsigned int samplesToCopyFromBeginning = samplesToWrite - samplesBeforeEndOfInput;
		memcpy(&destination[samplesBeforeEndOfInput], &sampleBuffer[0], samplesToCopyFromBeginning * sizeof(Sample));
	}
	else
	{
		// No wrapping, just copy.
		memcpy(&destination[0], &sampleBuffer[samplesWritten], samplesToWrite * sizeof(Sample));
	}

	// Update the write offset.
	samplesWritten = (samplesWritten + samplesToWrite) % bufferSize;
}

qboolean SNDDMA_Init(void)
{
	shm = &sn;
	
	shm->channels			= channelCount;
	shm->samplebits			= 16;
	shm->speed				= frequency;
	shm->soundalive			= qtrue;
	shm->splitbuffer		= qfalse;
	shm->samples			= bufferSize * channelCount;
	shm->samplepos			= 0;
	shm->submission_chunk	= 1;
	shm->buffer				= (unsigned char *) sampleBuffer;

	pspAudioInit();
	pspAudioSetChannelCallback(0, audioCallback, NULL);

	return qtrue;
}

void SNDDMA_Shutdown(void)
{
	pspAudioEnd();
}

int SNDDMA_GetDMAPos(void)
{
	return samplesWritten * channelCount;
}

void SNDDMA_Submit(void)
{
}
