/*
Quake PSP port.
Copyright (C) 2005 Peter Mackay

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>

#include <pspkernel.h>
#include <pspdebug.h>

extern "C"
{
#include "../sys.h"
#include "../quakedef.h"
}

int Sys_FileOpenRead (char *path, int *hndl)
{
	// Open the file.
	FILE* file = fopen(path, "rb");
	if (!file)
	{
		return -1;
	}

	// Get the length.
	fseek(file, 0, SEEK_END);
	const long length = ftell(file);
	fseek(file, 0, SEEK_SET);

	// Done.
	*hndl = reinterpret_cast<int> (file);
	return length;
}

int Sys_FileOpenWrite (char *path)
{
	FILE* const file = fopen(path, "wb");
	return reinterpret_cast<int> (file);
}

void Sys_FileClose (int handle)
{
	FILE* const file = reinterpret_cast<FILE*> (handle);
	fclose(file);
}

void Sys_FileSeek (int handle, int position)
{
	FILE* const file = reinterpret_cast<FILE*> (handle);
	fseek(file, position, SEEK_SET);
}

int Sys_FileRead (int handle, void *dest, int count)
{
	FILE* const file = reinterpret_cast<FILE*> (handle);
	return fread(dest, 1, count, file);
}

int Sys_FileWrite (int handle, void *data, int count)
{
	FILE* const file = reinterpret_cast<FILE*> (handle);
	return fwrite(data, 1, count, file);
}

int	Sys_FileTime (char *path)
{
	struct stat s;
	memset(&s, 0, sizeof(s));

	if (stat(path, &s) != 0)
	{
		return -1;
	}

	return s.st_ctime;
}

void Sys_mkdir (char *path)
{
	mkdir(path, 0x777);
}

void Sys_Error (char *error, ...)
{
	va_list args;
	va_start(args, error);
	char buffer[1024];
	memset(buffer, 0, sizeof(buffer));
	vsnprintf(buffer, sizeof(buffer) - 1, error, args);
	va_end(args);

	pspDebugScreenSetTextColor(0x0000ff);
	pspDebugScreenPrintData(buffer, strlen(buffer));
	
	sceKernelDelayThread(10000000);
	Sys_Quit();
}

void Sys_Printf (char *fmt, ...)
{
	char buffer[1024];

	va_list args;
	va_start(args, fmt);
	memset(buffer, 0, sizeof(buffer));
	vsnprintf(buffer, sizeof(buffer) - 1, fmt, args);
	va_end(args);

//	pspDebugScreenPrintData(buffer, strlen(buffer));
}

void Sys_Quit (void)
{
	Host_Shutdown();
	sceKernelExitGame();
}

double Sys_FloatTime (void)
{
	static double	runningTime		= 0.0f;
	static clock_t	lastTicks		= clock();

	const clock_t	ticks			= clock();
	const clock_t	deltaTicks		= ticks - lastTicks;
	const double	deltaSeconds	= deltaTicks * (1.0 / CLOCKS_PER_SEC);
	runningTime += deltaSeconds;

	return runningTime;
}

char *Sys_ConsoleInput (void)
{
	return 0;
}

void Sys_SendKeyEvents (void)
{
}

void Sys_LowFPPrecision (void)
{
}

void Sys_HighFPPrecision (void)
{
}
